//
// JTypist v3.1 - Java typing tutor program for UNIX systems
// Copyright (C) 2000  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// Gauge class - implements a Windows-style gauge GUI component
//
import java.awt.Color;
import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Frame;
import java.awt.BorderLayout;

/**
 * Implements a Windows-style gauge GUI component.
 */
public class Gauge extends Canvas {

	// the gauge current value, range, units, and preferred size
	private int		range_min	= 0;
	private int		range_max	= 100;
	private int		value		= 0;
	private String		units		= " %";
	private String		no_data		= "---";
	private Dimension	pref_size	= new Dimension( 40, 18 );

	// gauge active or inactive (no-data) flag
	private boolean		active		= true;

	// gauge bar colour, defaults to blue, arbitrarily
	private Color		bar_colour	= Color.blue;

	// gauge inactive background, light gray if nothing else given
	private Color		inactive_bgcolour
						= Color.lightGray;


	/**
	 * Create a gauge with default range, value, units, and no-data.
	 */
	public Gauge() {}

	/**
	 * Create a gauge with specified range, value, units and no-data
	 * string.
	 */
	public Gauge( int range_min, int range_max, int value, String units,
			String no_data ) {
		this.range_min	= range_min;
		this.range_max	= range_max;
		this.value	= value;
		this.units	= units;
		this.no_data	= no_data;
	}

	/*
	 * Return dimensions.
	 */
	public synchronized Dimension getPreferredSize() {
		 return ( pref_size ); }
	public synchronized Dimension getMinimumSize() {
		 return ( pref_size ); }

	/**
	 * Paint the graphic.  This method fills in the complete canvas
	 * area.
	 */
	public synchronized void paint( Graphics g ) {
		Dimension d = getSize();

		// if the gauge is active then create the bar effect
		if ( active ) {
			// calculate the fill width, and reset its bounds if
			// it could be outside the allowed area
			int fill_width = (( d.width - 4 )
						* ( value - range_min ))
						/ ( range_max - range_min );
			if ( fill_width < 0 )
				fill_width = 0;
			else if ( fill_width > d.width - 4 )
				fill_width = d.width - 4;

			// plot in the gauge value
			g.setColor( bar_colour );
			g.fillRect( 2, 2, fill_width, d.height - 4 );
			g.setColor( getBackground() );
			g.fillRect( fill_width + 1, 2, d.width - 4,
								d.height - 4 );

			// write the numeric value centrally in the gauge
			FontMetrics fm	= g.getFontMetrics();
			String svalue	= Integer.toString( value ) + units;
			g.setColor( getForeground() );
			g.drawString( svalue,
				( d.width - fm.stringWidth( svalue )) / 2,
				( d.height + fm.getAscent() ) / 2 );

		}

		else {
			// inactive gauge, so present a blank box
			g.setColor( inactive_bgcolour );
			g.fillRect( 2, 2, d.width - 4, d.height - 4 );

			// add the relevant 'nothing to say' indication
			FontMetrics fm	= g.getFontMetrics();
			g.setColor( getForeground() );
			g.drawString( no_data,
				( d.width - fm.stringWidth( no_data )) / 2,
				( d.height + fm.getAscent() ) / 2 );
		}

		// end with a small box round the gauge canvas
		g.setColor( getBackground() );
		g.draw3DRect( 0, 0, d.width - 1, d.height - 1, false );
	}

	/**
	 * Update the graphic.  Since no smart update is available, and
	 * since paint paints the complete graphic area, we get away with
	 * just calling paint().
	 */
	public synchronized void update( Graphics g ) { paint( g ); }


	/*
	 * Standard get/set methods.
	 */
	public synchronized int	get_min()	{ return ( range_min ); }
	public synchronized int	get_max()	{ return ( range_max ); }
	public synchronized int	get_value()	{ return ( value ); }
	public synchronized String get_units()	{ return ( units ); }
	public synchronized String get_no_data(){ return ( no_data ); }
	public synchronized Color get_colour()	{ return ( bar_colour ); }
	public synchronized Color get_inactive_colour()
						{ return ( inactive_bgcolour );}
	public synchronized boolean get_active(){ return ( active ); }
	public synchronized void set_min( int range_min ) {
		if ( range_min != this.range_min ) {
			this.range_min = range_min; repaint();
		}
	}
	public synchronized void set_max( int range_max ) {
		if ( range_max != this.range_max ) {
			this.range_max = range_max; repaint();
		}
	}
	public synchronized void set_value( int value ) {
		if ( value != this.value ) {
			this.value = value; repaint();
		}
	}
	public synchronized void set_units( String units ) {
		if ( units != this.units ) {
			this.units = units; repaint();
		}
	}
	public synchronized void set_no_data( String no_data ) {
		if ( no_data != this.no_data ) {
			this.no_data = no_data; repaint();
		}
	}
	public synchronized void set_colour( Color colour ) {
		if ( colour != bar_colour ) {
			this.bar_colour = colour; repaint();
		}
	}
	public synchronized void set_inactive_bgcolour( Color colour ) {
		if ( colour != inactive_bgcolour ) {
			this.inactive_bgcolour = colour; repaint();
		}
	}
	public synchronized void set_active( boolean active ) {
		if ( active != this.active ) {
			this.active = active; repaint();
		}
	}


	/**
	 * Class test code.
	 */
	public static void main( String args[] ) {
		System.out.println( "*** CLASS TEST CODE ***" );

		Frame frame = new Frame();
		Gauge gauge = new Gauge();
		gauge.set_colour( Color.red );
		gauge.set_inactive_bgcolour( Color.green );
		frame.setLayout( new BorderLayout() );
		frame.add( "Center", gauge );
		frame.pack(); frame.setVisible( true );
		for ( int i = -10; i <= 110; i++ ) {
			gauge.set_value( i );
			if ( i >=40 && i <= 70 )
				gauge.set_active( false );
			else
				gauge.set_active( true );
			try { Thread.sleep( 100 ); }
			catch( InterruptedException e ) {}
		}
	}
}
